import argparse
from joetags.add import add
from joetags.search import search_interface
from joetags.browse import browse


def main():
    # top level parser
    parser = argparse.ArgumentParser(prog='Joetags')
    parser.add_argument('-v', '--version', help='displays current version', action="store_true")
    subparsers = parser.add_subparsers(help='sub-command help')

    parser_add = subparsers.add_parser('add', aliases=['a'], description='add tags to dirs')
    parser_add.add_argument('directory', nargs='?', default="", help="subdir must be in currend dir")
    parser_add.set_defaults(func=add)

    parser_search = subparsers.add_parser('search', aliases=['s'], description="search for tags and groups by applying filters")
    parser_search.add_argument('filter', nargs='?', help="use tags or groups and combine with union(|), intersection(&) and negation(!), and brackets. put filter inside ''")
    parser_search.set_defaults(func=search_interface)

    parser_browse = subparsers.add_parser('ls', aliases=['browse', 'b'], description="browse for tags and groups to find what you are looking for or explore stuff")
    parser_browse.add_argument("-g", "--groups", action="store_true", help="get all groups in selection ignored when -c flag used")
    parser_browse.add_argument("-t", "--tags", action="store_true", help="get all tags in selection, ignored when -c flag used")
    parser_browse.add_argument("-c", "--use_groupname", action="store_true", help="selection will be groupname, -t and -g tags are ignored when used")
    parser_browse.add_argument("browse_selection", type=str, nargs='?', help="folder or groupname")
    parser_browse.set_defaults(func=browse)

    args = parser.parse_args()
    if args.version:
        print_version()
        exit()

    args.func(args)

def print_version():
    # for faster startup lazy import
    import pkg_resources
    version =  pkg_resources.require("joetags")[0].version
    print(f"joetags version {version}")

if __name__ == '__main__':
    main()



from joetags.core.build import build_tagmap


def browse(args):

    if args.use_groupname:
        if args.browse_selection != None:
            group = args.browse_selection
            tagmap = build_tagmap()
            tags = tagmap.group_tags(group)
            tagnames = list(map(lambda x: x.name, tags))
            # removes duplicates
            tagnames = list(set(tagnames))

            for tagname in tagnames:
                print(tagname)
        else:
            print("You have to specify a Group Name")
        exit()

    else:
        tagmap = build_tagmap()
        results = []
        if not args.groups and not args.tags:
            # when no group and not tag flag return both
            args.groups = True
            args.tags = True

        if args.groups == True:
            results.extend(tagmap.groups(args.browse_selection))
        if args.tags == True:
            results.extend(tagmap.tags(args.browse_selection))

        # removes all duplicates
        results = list(set(results))

        for result in results:
            print(result)

        exit()





import os
import subprocess


def add(args):
    '''Tags new Folders'''
    if args.directory == "":
        print("Nothing specified!\nPlease specify file")
        exit()

    else:
        directory = args.directory

        if not os.path.exists(directory):
            print(f"fatal: pathspec '{directory}' did not match any directories")
            exit()

        if not os.access(directory, os.W_OK):
            print(f"fatal: no writing permissions in '{directory}'")
            exit()

        vim_add(directory)
    
def vim_add(directory):
    subprocess.run(f"vim + ./{directory}/.joetag",shell=True)




from joetags.core.build import build_tagmap
from joetags.helper.colorprint import wrap_color


def search_interface(args):

    if args.filter == None:
        tagfilter = input("Enter Filter:")
    else:
        tagfilter = args.filter

    search(tagfilter)

    
def search(tagfilter):
    tagmap = build_tagmap()
    results = tagmap.search(tagfilter)
    if len(results) > 0 : 
        print(wrap_color("Results", "green"))
        for result in results:
            print(wrap_color("   " + result, "green"))

    else : print("no results found")
    
    print(wrap_color(tagmap.warning(), "red"))




import setuptools
from pathlib import Path

setuptools.setup(
    name = 'joetags',
    version = '0.2.1',
    description=('tag and organize your projects'),
    long_description=Path("README.md").read_text(),
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    maintainer="realsirjoe",
    author="realsirjoe",
    entry_points = {
        'console_scripts': [
            'joetags = joetags.__main__:main',
            'jt = joetags.__main__:main'
        ]
    })
